@@ -9,7 +9,6 @@ import os
 from sys import version_info
 from matplotlib import pyplot as plt
 
-from astropy.io import fits as pf
 from argparse import ArgumentParser
 
 from .dataset import DataSet
@@ -33,23 +32,6 @@ with open(os.path.join(code_dir, 'VERSION')) as version_file:
     __version__ = version
 
 
-def air2vac(air):
-    """
-    Air to vacuum conversion from Bengt Edlén 1953,
-    Journal of the Optical Society of America, Vol. 43, Issue 5, pp. 339-344.
-    """
-    if np.min(air) < 2000.:
-        raise ValueError("Input wavelength below valid range!")
-    air = np.array(air)
-    ij = (np.array(air) >= 2000)
-    out = np.array(air).copy()
-    s2 = (1.e4/air)**2
-    fact = 1.0 + 6.4328e-5 + 2.94981e-2/(146.0 - s2) + 2.5540e-4/(41.0 - s2)
-    # Alternative solution from VALD:
-    # fact = 1 + 8.336624e-5 + 0.02408927 / (130.106592 - s2) + 1.599740895e-4 / (38.925688 - s2),
-    out[ij] = air[ij]*fact[ij]
-    return out
-
 
 def main():
 
@@ -67,7 +49,9 @@ def main():
     print(r"                \/                     ")
     print(r"")
     print(r"")
-    print(r" Loaded Solar abundances from Asplund et al. 2009  (photospheric)")
+    print(r" Loaded Solar abundances from Asplund et al. (2009)")
+    print(r" Using recommendations from Lodders et al. (2009)")
+    print(r" regarding the source: photospheric, meteoritic or their average.")
     print(r"")
 
     descr = """VoigtFit Absorption Line Fitting.
@@ -101,56 +85,23 @@ def main():
     if os.path.exists(name+'.hdf5') and not args.f:
         dataset = load_dataset(name+'.hdf5')
 
-        # if len(dataset.data) != len(parameters['data']):
-        dataset.data = list()
-        # Setup data:
-
-        for fname, res, norm, airORvac, nsub in parameters['data']:
-            if verbose:
-                print(" Loading data: " + fname)
-
-            if fname[-5:] == '.fits':
-                hdu = pf.open(fname)
-                spec = pf.getdata(fname, 0)
-                hdr = pf.getheader(fname)
-                wl = hdr['CRVAL1'] + np.arange(len(spec))*hdr['CD1_1']
-
-                if len(hdu) > 1:
-                    err = pf.getdata(fname, 1)
-                elif parameters['snr'] is not None:
-                    # err = spec/parameters['snr']
-                    err = np.ones_like(spec)*np.median(spec)/parameters['snr']
-                else:
-                    err = spec/10.
-                err[err <= 0.] = np.abs(np.mean(err))
-                mask = np.ones_like(wl, dtype=bool)
-
-            else:
-                data = np.loadtxt(fname)
-                if data.shape[1] == 2:
-                    wl, spec = data.T
-                    if parameters['snr'] is not None:
-                        # err = spec/parameters['snr']
-                        err = np.ones_like(spec)*np.median(spec)/parameters['snr']
-                    else:
-                        err = spec/10.
-                    err[err <= 0.] = np.abs(np.mean(err))
-                    mask = np.ones_like(wl, dtype=bool)
-                elif data.shape[1] == 3:
-                    wl, spec, err = data.T
-                    mask = np.ones_like(wl, dtype=bool)
-                elif data.shape[1] == 4:
-                    wl, spec, err, mask = data.T
-
-            if airORvac == 'air':
+        all_spectra_defined = all([data_item[0] in dataset.data_filenames for data_item in parameters['data']])
+        match_number_of_spectra = len(dataset.data) == len(parameters['data'])
+        if match_number_of_spectra and all_spectra_defined:
+            # all data is already defined in the dataset
+            # do nothing and just move on to setting up lines and components
+            pass
+        else:
+            dataset.data = list()
+            for fname, res, norm, airORvac, nsub, ext, use_mask in parameters['data']:
                 if verbose:
-                    print(" Converting wavelength from air to vacuum.")
-                wl = air2vac(wl)
+                    print(" Loading data: " + fname)
 
-            dataset.add_data(wl, spec, res,
-                             err=err, normalized=norm, mask=mask, nsub=nsub)
-            if verbose:
-                print(" Successfully added spectrum to dataset.\n")
+                dataset.add_spectrum(fname, res, airORvac, verbose=verbose, ext=ext, normalized=norm, nsub=nsub, use_mask=use_mask)
+                if verbose:
+                    print(" Successfully added spectrum to dataset.\n")
+            # Reset all the regions in the dataset to force-reload the data:
+            dataset.reset_all_regions()
 
         # -- Handle `lines`:
         # Add new lines that were not defined before:
@@ -214,7 +165,6 @@ def main():
         if verbose:
             print("\n - Fine-structure lines in dataset:")
             print(dataset.fine_lines)
-            print(list(dataset.lines.keys()))
             print("\n - Fine-structure lines in parameter file:")
             print(parameters['fine-lines'])
         if len(parameters['fine-lines']) > 0:
@@ -286,54 +236,20 @@ def main():
                 if band not in defined_molecular_bands:
                     dataset.deactivate_molecule(molecule, band)
 
-    # -- Otherwise create a new DataSet
+
     else:
-        # ================================================================================
-        # Generate New Dataset:
-        #
+        # --- Create a new DataSet
         dataset = DataSet(parameters['z_sys'], parameters['name'])
 
         if 'velspan' in parameters.keys():
             dataset.velspan = parameters['velspan']
 
-        # Setup data:
-        for fname, res, norm, airORvac, nsub in parameters['data']:
-            if fname[-5:] == '.fits':
-                hdu = pf.open(fname)
-                spec = pf.getdata(fname)
-                hdr = pf.getheader(fname)
-                wl = hdr['CRVAL1'] + np.arange(len(spec))*hdr['CD1_1']
-                if len(hdu) > 1:
-                    err = hdu[1].data
-                elif parameters['snr'] is not None:
-                    # err = spec/parameters['snr']
-                    err = np.ones_like(spec)*np.median(spec)/parameters['snr']
-                else:
-                    err = spec/10.
-                err[err <= 0.] = np.abs(np.mean(err))
-                mask = np.ones_like(wl, dtype=bool)
+        # Load data:
+        for fname, res, norm, airORvac, nsub, ext, use_mask in parameters['data']:
+            if verbose:
+                print(" Loading data: " + fname)
 
-            else:
-                data = np.loadtxt(fname)
-                if data.shape[1] == 2:
-                    wl, spec = data.T
-                    if parameters['snr'] is not None:
-                        # err = spec/parameters['snr']
-                        err = np.ones_like(spec)*np.median(spec)/parameters['snr']
-                    else:
-                        err = spec/10.
-                    err[err <= 0.] = np.abs(np.mean(err))
-                    mask = np.ones_like(wl, dtype=bool)
-                elif data.shape[1] == 3:
-                    wl, spec, err = data.T
-                    mask = np.ones_like(wl, dtype=bool)
-                elif data.shape[1] == 4:
-                    wl, spec, err, mask = data.T
-
-            if airORvac == 'air':
-                wl = air2vac(wl)
-
-            dataset.add_data(wl, spec, res, err=err, normalized=norm, mask=mask, nsub=nsub)
+            dataset.add_spectrum(fname, res, airORvac, verbose=verbose, ext=ext, normalized=norm, nsub=nsub, use_mask=use_mask)
 
         # Define lines:
         for tag, velspan in parameters['lines']:
@@ -352,21 +268,20 @@ def main():
     # =========================================================================
     # Back to Common Work Flow for all datasets:
 
-    # HERE masking is correct!
     dataset.verbose = verbose
 
-    # Load components from file:
-    if 'load' in parameters.keys():
-        dataset.reset_components()
-        for fname in parameters['load']:
-            print("\n Loading parameters from file: %s \n" % fname)
-            dataset.load_components_from_file(fname)
-    else:
-        dataset.reset_components()
+    if len(parameters['limits']) > 0:
+        for limit_lines, limit_options in parameters['limits']:
+            dataset.add_lines(limit_lines, velspan=dataset.velspan)
+            for line_tag in limit_lines:
+                dataset.deactivate_line(line_tag)
 
-    # Fix the velocity structure of the loaded lines:
-    if parameters['fix_velocity']:
-        dataset.fix_structure()
+    # Load components from file:
+    dataset.reset_components()
+    if 'load' in parameters.keys():
+        for fname in parameters['load']:
+            print("\n Loading components from file: %s \n" % fname)
+            dataset.load_components_from_file(fname)
 
     # Prepare thermal model infrastructure:
     if len(parameters['thermal_model']) > 0:
@@ -452,6 +367,10 @@ def main():
                 if num in thermal_model[ion]:
                     thermal_model[ion].remove(num)
 
+    # Fix the velocity structure of the loaded lines:
+    if parameters['fix_velocity']:
+        dataset.fix_structure()
+
     # Set default value of norm:
     norm = False
     if 'cheb_order' in parameters.keys():
@@ -485,6 +404,12 @@ def main():
         print(stat_msg % (order_str))
         print("")
 
+        if len(parameters['limits']) > 0:
+            print("\n\n* In order to determine the equivalent width, you must normalize the line(s)")
+            for limit_lines, _ in parameters['limits']:
+                for line_tag in limit_lines:
+                    dataset.normalize_line(line_tag, norm_method=dataset.norm_method)
+
     # Parse show_vel_norm from parameter file:
     # Ketyword 'norm_view' either vel or wave.
     if 'vel' in parameters['norm_view'].lower():
@@ -494,22 +419,15 @@ def main():
     else:
         show_vel_norm = False
 
-    # Reset data in regions:
-    # This keyword is deprecated and will be removed shortly!!
-    if 'reset' in parameters.keys():
-        if len(parameters['reset']) > 0:
-            for line_tag in parameters['reset']:
-                regions_of_line = dataset.find_line(line_tag)
-                for reg in regions_of_line:
-                    dataset.reset_region(reg)
-        else:
-            dataset.reset_all_regions()
 
     # prepare_dataset
     if verbose:
         print(" - Preparing dataset:")
-    dataset.prepare_dataset(mask=False, norm=norm, velocity=show_vel_norm,
-                            **parameters['check_lines'])
+    prep_msg = dataset.prepare_dataset(mask=False, norm=norm, velocity=show_vel_norm,
+                                       **parameters['check_lines'])
+    if prep_msg and not dataset.ready2fit:
+        print(prep_msg)
+        return False
 
     # Define thermal model
     if len(thermal_model.keys()) > 0:
@@ -555,9 +473,9 @@ def main():
         show_vel_mask = False
 
     # Mask invidiual lines
-    if verbose:
-        print(" Masking parameters:", parameters['mask'])
     if 'mask' in parameters.keys():
+        if verbose:
+            print(" Masking parameters:", parameters['mask'])
         if len(parameters['mask']) > 0:
             for line_tag, reset in zip(parameters['mask'], parameters['forced_mask']):
                 dataset.mask_line(line_tag, reset=reset,
@@ -579,12 +497,20 @@ def main():
             dataset.set_resolution(item[0], item[1])
 
     # Run the fit:
+    print("\n  Fit is running... Please be patient.\n")
     popt, chi2 = dataset.fit(verbose=False, **parameters['fit_options'])
 
     print(" The fit has finished with the following exit message:")
     print("  " + popt.message)
     print("")
 
+
+    for reg in dataset.regions:
+        print(reg.lines)
+        print("Norm?  %r" % reg.normalized)
+        print("")
+
+
     # Fix for when the code cannot estimate uncertainties:
     for parname in dataset.best_fit.keys():
         err = dataset.best_fit[parname].stderr
@@ -635,9 +561,6 @@ def main():
         # print Thermal Model Parameters
         output.print_T_model_pars(dataset, thermal_model)
 
-    if dataset.cheb_order >= 0:
-        dataset.print_cont_parameters()
-
     # print metallicity
     logNHI = parameters['logNHI']
     if 'HI' in dataset.components.keys():
@@ -649,6 +572,23 @@ def main():
     if parameters['show_total']:
         dataset.print_total()
 
+    filename = name
+    # determine limits, if any
+    if len(parameters['limits']) > 0:
+        print("\n\n---------------------------")
+        print(" Determining Upper Limits:")
+        print("---------------------------")
+        EW_limits = list()
+        for limit_lines, limit_options in parameters['limits']:
+            for line_tag in limit_lines:
+                EW = dataset.equivalent_width_limit(line_tag, verbose=True, **limit_options)
+                if EW is not None:
+                    EW_limits.append(EW)
+                    print(output.format_EW(EW))
+        print("")
+        # Save to file:
+        output.save_EW(EW_limits, filename+'.limits')
+
     # Output:
     if 'individual-regions' in parameters['output_pars']:
         individual_regions = True
@@ -660,19 +600,19 @@ def main():
     else:
         individual_components = False
 
-    filename = name
     # plot and save
     if 'rebin' in parameters['fit_options'].keys():
         rebin = parameters['fit_options']['rebin']
     else:
         rebin = 1
-    dataset.plot_fit(filename=filename, rebin=rebin, subsample_profile=3)
+    dataset.plot_fit(filename=filename, rebin=rebin)
     output.save_parameters_to_file(dataset, filename+'.fit')
     output.save_cont_parameters_to_file(dataset, filename+'.cont')
     output.save_fit_regions(dataset, filename+'.reg',
                             individual=individual_regions)
     if individual_components:
         output.save_individual_components(dataset, filename+'.components')
+    print("  Done...\n")
     plt.show(block=True)
 
 
